Exception: WPScan::Error::SAMLAuthenticationRequired

Inherits:
Standard
  • Object
show all
Defined in:
lib/wpscan/errors/saml.rb

Overview

SAML Authentication Required Error

Instance Method Summary collapse

Constructor Details

#initialize(message = 'SAML authentication is required to access this resource, consider using --expect-saml.') ⇒ SAMLAuthenticationRequired

Returns a new instance of SAMLAuthenticationRequired.



7
8
9
# File 'lib/wpscan/errors/saml.rb', line 7

def initialize(message = 'SAML authentication is required to access this resource, consider using --expect-saml.')
  super
end