Class: WPScan::Finders::DynamicFinder::Version::Finder

Inherits:
Finder
  • Object
show all
Defined in:
lib/wpscan/finders/dynamic_finder/version/finder.rb

Overview

To be used as a base when creating a dynamic finder to find the version of a WP Item (such as theme/plugin)

Constant Summary

Constants inherited from Finder

Finder::DIRECT_ACCESS

Instance Attribute Summary

Attributes inherited from Finder

#progress_bar, #target

Method Summary

Methods inherited from Finder

#aggressive, child_class_constant, child_class_constants, create_child_class, #find, #passive

Methods inherited from Finder

#aggressive, #browser, #create_progress_bar, #found_by, #hydra, #initialize, #passive, #titleize

Constructor Details

This class inherits a constructor from WPScan::Finders::Finder