Module: WPScan::Finders::IndependentFinder

Extended by:
ActiveSupport::Concern
Included in:
WPScan::Finders::InterestingFindings::Base
Defined in:
lib/wpscan/finders/independent_finder.rb

Overview

Independent Finder

Defined Under Namespace

Modules: ClassMethods

Instance Method Summary collapse

Instance Method Details

#find(opts = {}) {|Finding| ... } ⇒ Findings

Parameters:

  • opts (Hash) (defaults to: {})

Options Hash (opts):

  • mode (Symbol) — default: :mixed, :passive, :aggressive

Yields:

  • (Finding)

    Optional block called for each finding the moment it is first appended to the result set (used to stream enumeration findings as they are discovered).

Returns:



23
24
25
# File 'lib/wpscan/finders/independent_finder.rb', line 23

def find(opts = {}, &)
  finders.run(opts, &)
end

#findersArray

Returns:

  • (Array)


28
29
30
# File 'lib/wpscan/finders/independent_finder.rb', line 28

def finders
  @finders ||= WPScan::Finders::IndependentFinders.new
end