Class: WPScan::Finders::Plugins::QueryParameter

Inherits:
DynamicFinder::WpItems::Finder show all
Defined in:
app/finders/plugins/query_parameter.rb

Overview

Plugins finder from Dynamic Finder ‘QueryParameter’

Constant Summary collapse

DEFAULT_CONFIDENCE =
10

Constants inherited from Finder

Finder::DIRECT_ACCESS

Instance Attribute Summary

Attributes inherited from Finder

#progress_bar, #target

Instance Method Summary collapse

Methods inherited from DynamicFinder::WpItems::Finder

#aggressive, #aggressive_, #aggressive_configs, #aggressive_path, #passive_configs

Methods inherited from Finder

#aggressive, #browser, #create_progress_bar, #found_by, #hydra, #initialize, #titleize

Constructor Details

This class inherits a constructor from WPScan::Finders::Finder

Instance Method Details

#passive(_opts = {}) ⇒ Object



10
11
12
# File 'app/finders/plugins/query_parameter.rb', line 10

def passive(_opts = {})
  # Handled by UrlsInHomePage, so no need to check this twice
end

#process_response(opts, response, slug, klass, config) ⇒ Plugin

Returns The detected plugin in the response, related to the config.

Parameters:

  • opts (Hash)

    The options from the #passive, #aggressive methods

  • response (Typhoeus::Response)
  • slug (String)
  • klass (String)
  • config (Hash)

    The related dynamic finder config hash

Returns:

  • (Plugin)

    The detected plugin in the response, related to the config



21
22
23
# File 'app/finders/plugins/query_parameter.rb', line 21

def process_response(opts, response, slug, klass, config)
  # TODO: when a real case will be found
end