Exception: WPScan::Error::SAMLAuthenticationRequired
- Defined in:
- lib/wpscan/errors/saml.rb
Overview
SAML Authentication Required Error
Instance Method Summary collapse
-
#initialize(message = 'SAML authentication is required to access this resource, consider using --expect-saml.') ⇒ SAMLAuthenticationRequired
constructor
A new instance of SAMLAuthenticationRequired.
Constructor Details
#initialize(message = 'SAML authentication is required to access this resource, consider using --expect-saml.') ⇒ SAMLAuthenticationRequired
Returns a new instance of SAMLAuthenticationRequired.
7 8 9 |
# File 'lib/wpscan/errors/saml.rb', line 7 def initialize( = 'SAML authentication is required to access this resource, consider using --expect-saml.') super end |